#!/bin/sh
# Begin libiptables.sh

include common
include color
include pretty

depends iptables

[ "$AUTH_PREFIX" == "" ] && AUTH_PREFIX="AUTH-"
[ "$DROP_PREFIX" == "" ] && DROP_PREFIX="DROP-"
[ "$MARK_PREFIX" == "" ] && MARK_PREFIX="MARK-"
[ "$JUMP_PREFIX" == "" ] && JUMP_PREFIX="JUMP-"
[ "$DNAT_PREFIX" == "" ] && DNAT_PREFIX="DNAT-"
[ "$LOG_PREFIX" == ""  ] &&  LOG_PREFIX="LOG-"

function default_policy
{

    # TODO : should try to detect if some of this isn't already done.

    if matches "$1" '/\(ACCEPT\|DROP\)/'; then
	pol=$1;
    else
	matches "$POLICY" '/\(ACCEPT\|DROP\)/' && pol=$POLICY || pol="ACCEPT" 
    fi

    Section "Setting default policy"

    for i in INPUT OUTPUT FORWARD; do
	Elt "Set ${YELLOW}$i${NORMAL} default policy to ${YELLOW}$pol${NORMAL}"
	if iptables -L | grep "^Chain $i (policy $pol)\$" >/dev/null 2>&1 ; then
	    print_info "Already done"
	    print_status noop
	else
	    iptables -P $i $pol
	    Feedback
	fi
    done
}


function table_clean_all()
{
    
    # TODO : should try to detect if some of this isn't already done.

    default_policy ACCEPT

    Section "Cleaning Tables"

    Elt "Generals" ; iptables -F ; Feedback
    Elt "NAT" ; iptables -t nat -F ; Feedback
    Elt "Mangle" ; iptables -t mangle -F ; Feedback
    Elt "User tables" ; iptables -X ; Feedback

}



function exist_chain()
{
	usage_string="usage: $FUNCNAME {chain} [table]"

        [ "$#" != "1" ] && [ "$#" != "2" ] && print_exit $usage_string

	[ "$2" == "" ] && table="" || table="-t $2"

	iptables $table -L $1 >/dev/null 2>&1
	return $?
};

function exist_table()
{
	usage_string="usage: $FUNCNAME {table}"

        [ "$#" != "1" ] && print_exit $usage_string
	
	iptables -t $1 -L > /dev/null 2>&1
	return $?
};

function init_table()
{
	usage_string="usage: $FUNCNAME {table} [tables ...]"

        [ "$#" == "0" ] && print_exit $usage_string
	
	for i in $*
	do
	  iptables -t $i -F > /dev/null 2>&1 || print_error "'$i' is not a recognized table"
	done
};

function BEGIN_TABLE()
{
    usage_string="usage: $FUNCNAME {table}"

    [ "$#" == 1 ] || print_exit $usage_string
    
    [ "$Current_table" == "" ] ||
        print_syntax_error "$FUNCNAME $1 without END_TABLE $Current_table"

    exist_table $1 ||
        print_syntax_error "$FUNCNAME $1 : '$1' is not recognized."

    Current_table="$1"
}

function END_TABLE()
{
    usage_string="usage: $FUNCNAME {table}"

    [ "$#" == 1 ] || print_exit $usage_string

    [ "$Current_table" == "$1" ] ||
        print_syntax_error "$FUNCNAME $1 without BEGIN_TABLE $1"

    Current_table=""
}

function BEGIN_CHAIN()
{
    usage_string="usage: $FUNCNAME {chain}"

    [ "$#" == 1 ] || print_exit $usage_string
    
    [ "$Current_table" != "" ] ||
        print_syntax_error "$FUNCNAME $1 with no previous BEGIN_TABLE $Current_table"
    [ "$Current_chain" == "" ] ||
        print_syntax_error "$FUNCNAME $1 with no previous END_CHAIN $Current_chain"

    exist_chain $1 ${Current_table} ||
        print_syntax_error "$FUNCNAME $1 : '$1' is not recognized in ${Current_table}."

    Current_chain="$1"
}

function END_CHAIN()
{
    usage_string="usage: $FUNCNAME {chain}"

    [ "$#" == 1 ] || print_exit $usage_string

    [ "$Current_table" != "" ] ||
        print_syntax_error "$FUNCNAME $1 with no previous BEGIN_TABLE $Current_table"
    [ "$Current_chain" == "$1" ] || 
        print_syntax_error "$FUNCNAME $1 with no previous BEGIN_CHAIN $1"

    Current_chain=""
}

function jump()
{
    usage_string="usage: $FUNCNAME [log] {target} [conditions]"

    [ "$#" != 0 ] || print_exit $usage_string

    [ "$Current_chain" != "" ] ||
        print_syntax_error "$FUNCNAME within no TABLE or CHAIN"

    log_act=""
    log_yesno=""
    if [ "$1" == "-log" ] || [ "$1" == "-nolog" ]
    then
	log_yesno="$1"
	log_prefix="$2"
	shift
	shift
    fi

    jump_to="$1"

    shift

    [ "$log_yesno" == "-log" ] &&
    iptables -t ${Current_table} -A ${Current_chain} $@ -m limit --limit 3/s -j LOG --log-prefix "$JUMP_PREFIX$log_prefix: "

    iptables -t ${Current_table} -A ${Current_chain} $@ -j $jump_to ||
    print_error "iptable returned error."
}

function soft_drop()
{
    usage_string="usage: $FUNCNAME [log] [conditions]"

#    [ "$#" != 0 ] || print_exit $usage_string

    [ "$Current_chain" != "" ] ||
        print_syntax_error "$FUNCNAME within no TABLE or CHAIN"

    if [ "$1" == "-log" ] || [ "$1" == "-nolog" ]
    then
	log_yesno="$1"
	log_prefix="$2"
	shift
	shift
	[ "$log_yesno" == "-log" ] &&
	iptables -t ${Current_table} -A ${Current_chain} $@ -m limit --limit 3/s -j LOG --log-prefix "$DROP_PREFIX$log_prefix: "
    fi

    iptables -t ${Current_table} -A ${Current_chain} $@ -m limit --limit 3/s -j REJECT &&
    iptables -t ${Current_table} -A ${Current_chain} $@ -j DROP ||
    print_error "iptable returned error."
}

function drop()
{
    usage_string="usage: $FUNCNAME [log] [conditions]"

#    [ "$#" != 0 ] || print_exit $usage_string

    [ "$Current_chain" != "" ] ||
        print_syntax_error "$FUNCNAME within no TABLE or CHAIN"

    if [ "$1" == "-log" ] || [ "$1" == "-nolog" ]
    then
	log_yesno="$1"
	log_prefix="$2"
	shift
	shift
	[ "$log_yesno" == "-log" ] &&
	iptables -t ${Current_table} -A ${Current_chain} $@ -m limit --limit 3/s -j LOG --log-prefix "$DROP_PREFIX$log_prefix: "
    fi

    iptables -t ${Current_table} -A ${Current_chain} $@ -j DROP ||
    print_error "iptable returned error."
}

function accept()
{
    usage_string="usage: $FUNCNAME [log] [conditions]"

    #[ "$#" != 0 ] || print_exit $usage_string

    [ "$Current_chain" != "" ] ||
        print_syntax_error "$FUNCNAME within no TABLE or CHAIN"

    log_yesno=""
    if [ "$1" == "-log" ] || [ "$1" == "-nolog" ]
    then
	log_yesno="$1"
	log_prefix="$2"
	shift
	shift
	[ "$log_yesno" == "-log" ] &&
	iptables -t ${Current_table} -A ${Current_chain} $@ -m limit --limit 3/s -j LOG --log-prefix "$AUTH_PREFIX$log_prefix: "
    fi

    iptables -t ${Current_table} -A ${Current_chain} $@ -j ACCEPT ||
    print_error "iptable returned error."
}

function mark()
{
    usage_string="usage: $FUNCNAME [log] {mark-stamp} [conditions]"

    [ "$#" != 0 ] || print_exit $usage_string

    [ "$Current_chain" != "" ] ||
        print_syntax_error "'$FUNCNAME' within no TABLE or CHAIN"

    [ "$Current_table" == "mangle" ] ||
        print_syntax_error "'$FUNCNAME' must be in mangle table"

    log_yesno=""
    if [ "$1" == "-log" ] || [ "$1" == "-nolog" ]
    then
	log_yesno="$1"
	log_prefix="$2"
	shift
	shift
    fi

    mark_stamp="$1"
    shift


    [ "$log_yesno" == "-log" ] &&
    iptables -t ${Current_table} -A ${Current_chain} $@ -m limit --limit 3/s -j LOG --log-prefix "$MARK_PREFIX$log_prefix: "

    iptables -t ${Current_table} -A ${Current_chain} $@ -j MARK --set-mark $mark_stamp ||
    print_error "iptable returned error."
}

function rule()
{
    usage_string="usage: $FUNCNAME [iptables string]"

    [ "$#" != 0 ] || print_exit $usage_string

    [ "$Current_chain" != "" ] ||
        print_syntax_error "'$FUNCNAME' within no TABLE or CHAIN"


#    if [ "$1" == "-log" ]
#    then
#	log_prefix="$2"
#	shift
#	shift
#	iptables -t ${Current_table} -A ${Current_chain} $@ -m limit --limit 3/s -j LOG --log-prefix $log_prefix
#    fi


    iptables -t ${Current_table} -A ${Current_chain} $@ -j MARK --set-mark $mark_stamp ||
    print_error "iptable returned error."
}

function log()
{
    usage_string="usage: $FUNCNAME {log-prefix} [conditions]"

    [ "$#" != 0 ] || print_exit $usage_string

    [ "$Current_chain" != "" ] ||
        print_syntax_error "'$FUNCNAME' within no CHAIN"

    log_prefix="$1"
    shift

    iptables -t ${Current_table} -A ${Current_chain} $@ -m limit --limit 3/s -j LOG --log-prefix "$LOG_PREFIX$log_prefix: " ||
    print_error "iptable returned error."
}

function dnat()
{
    usage_string="usage: $FUNCNAME [log] {dest} [conditions]"

    [ "$#" != 0 ] || print_exit $usage_string

    [ "$Current_chain" != "" ] ||
        print_syntax_error "'$FUNCNAME' within no TABLE or CHAIN"

    [ "$Current_table" == "nat" ] ||
        print_syntax_error "'$FUNCNAME' must be in nat table"

    log_yesno=""
    if [ "$1" == "-log" ] || [ "$1" == "-nolog" ]
    then
	log_yesno="1"
	log_prefix="$2"
	shift
	shift
    fi

    dest_ip="$1"
    shift


    [ "$log_yesno" == "-log" ] &&
    iptables -t ${Current_table} -A ${Current_chain} $@ -m limit --limit 3/s -j LOG --log-prefix "$MARK_DNAT$log_prefix: "

    iptables -t ${Current_table} -A ${Current_chain} $@ -j DNAT --to-destination $dest_ip ||
    print_error "iptable returned error."
}


# End libiptables.sh

